from napoleontoolbox.utility import date_utility
import pandas as pd


def indice_rebalancing(price_data= None, inception_date=None, target_weights=None, rebalancing_method = None):
    constituents = target_weights.keys()
    assert len([constituent for constituent in constituents if constituent in price_data.columns]) == len(constituents)
    assert sum(target_weights.values()) == 1.
    print(f'backtesting with {rebalancing_method} method rebalancing since {inception_date}')
    print(f'date range before filtering {min(price_data.index)} {max(price_data.index)}')
    price_data = price_data.loc[price_data.index >= inception_date, :]
    print(f'date range after filtering {min(price_data.index)} {max(price_data.index)}')
    assert inception_date in price_data.index

    date_utility.add_rebalancing_datepart(price_data, 'Date', rebalancing_method = rebalancing_method)
    assert price_data.shape[0] > 0

    initial_weights = target_weights.copy()
    initial_weights.update({
        'Date' : inception_date,
        'indice': 1.
    })
    weights_list = []
    weights_list.append(initial_weights)

    previous_weights = None
    previous_prices = None

    for date_index, row in price_data.iterrows():
        if date_index == inception_date :
            previous_weights = target_weights.copy()
            previous_prices = price_data.loc[inception_date].to_dict()
            previous_value = sum(previous_weights.values())
            continue
        current_weights = {}
        current_prices = price_data.loc[date_index].to_dict()
        for constituent in constituents:
            if row['is_rebalancing']:
                #### rebalancing to match the target : the actual previous weight is replaced by the total value equally sep
                current_weights[constituent] = previous_value * target_weights[constituent] * current_prices[constituent]/previous_prices[constituent]
            else:
                ##### drifting
                current_weights[constituent] = previous_weights[constituent] * current_prices[constituent]/previous_prices[constituent]

        current_value = sum(current_weights.values())
        current_weights.update({'Date' : date_index, 'indice': current_value})
        weights_list.append(current_weights)

        previous_weights = current_weights.copy()
        previous_prices = current_prices.copy()
        previous_value = current_value


    weights_df = pd.DataFrame(weights_list)
    return weights_df



from __future__ import print_function

def init_napoleon_colab():
    from napoleontoolbox.signal import signal_utility
    from napoleontoolbox.analyzer import minutely_market, hourly_market, market
    from datetime import datetime
    import pandas as pd
    from napoleontoolbox.file_saver import dropbox_file_saver

    from ipywidgets import interact, interactive, fixed, interact_manual
    import ipywidgets as widgets
    from napoleontoolbox.utility import metrics

    from pathlib import Path

    import matplotlib.pyplot as plt


    import plotly.offline as py
    py.init_notebook_mode(connected=True)
    import plotly.graph_objs as go
    import plotly
    from plotly.offline import iplot

    import os
    stream = os.popen('pip install dropbox')
    output = stream.read()
    print(output)

    stream = os.popen('pip install plotly==4.7.1')
    output = stream.read()
    print(output)

    def configure_plotly_browser_state():
      import IPython
      display(IPython.core.display.HTML('''
            <script src="/static/components/requirejs/require.js"></script>
            <script>
              requirejs.config({
                paths: {
                  base: '/static/base',
                  plotly: 'https://cdn.plot.ly/plotly-latest.min.js?noext',
                },
              });
            </script>
            '''))
    import IPython

    configure_plotly_browser_state()
    IPython.get_ipython().events.register('pre_run_cell', configure_plotly_browser_state)


def plot_dataframe(data):
    import plotly.graph_objs as go
    import plotly
    from plotly.offline import iplot
    data_list = []
    for me_commo in data.columns:
        trace_sig = go.Scatter(
            x=data.index,
            y=data[me_commo],
            name=me_commo,
            opacity=0.8)
        data_list.append(trace_sig)
    iplot(data_list)


from napoleontoolbox.forecasting import preprocessing_features
import numpy as np
import pandas as pd

class BaselineModel():

    def __init__(self, threshold = 0.7, signal_type = 'ls', mixing_type='sum', number_per_year=252):
        self.model = 'simple_model'
        self.threshold = threshold
        self.signal_type = signal_type
        self.mixing_type = mixing_type
        # self.output_ways = []
        # self.last_avg_return = pd.DataFrame()
        self.median_values = []
        self.way_last_five_avg_return = None
        self.way_last_return = None
        self.weigth_factors = None


    def fit(self, X_train, y_train, X_val, y_val):#, method = 'standard'):
        self.columns_to_drop = preprocessing_features.most_correlated_features(X_train, self.threshold)
        self.way_last_five_avg_return =y_train[-5:].mean()
        self.way_last_return = np.sign(y_train[-1])
        X_train = X_train.drop(self.columns_to_drop, axis=1)
        X_val = X_val.drop(self.columns_to_drop, axis=1)
        self.median_values = list(X_train.median().values)
        correlation_to_return = preprocessing_features.features_correlation_to_return(X_train,y_train)
        try:
            self.weigth_factors = preprocessing_features.rescalize(correlation_to_return, [min(correlation_to_return), max(correlation_to_return)],[-1, 1])
        except:
            self.weigth_factors = correlation_to_return.copy()
        for i in range(len(self.weigth_factors)):
            if self.way_last_five_avg_return >=0 and self.way_last_return>=0:
                # print('case 1')
                if self.weigth_factors[i]>=0:
                    None
                else:
                    self.weigth_factors[i] = -1*self.weigth_factors[i]
            elif self.way_last_five_avg_return <=0 and self.way_last_return<=0:
                # print('case 2')
                if self.weigth_factors[i] >= 0:
                    self.weigth_factors[i] = -1 * self.weigth_factors[i]
                else:
                    None
            else:
                if self.way_last_five_avg_return >=0 and self.way_last_return<=0:
                    # print('case 3')
                    if self.weigth_factors[i] >= 0:
                        None
                    else:
                        self.weigth_factors[i] = -1 * self.weigth_factors[i]
                else:
                    # print('case 4')
                    # print(self.weigth_factors)
                    if self.weigth_factors[i] >= 0:
                        self.weigth_factors[i] = -1 * self.weigth_factors[i]
                    else:
                        None
        # for daily_feat in X_train.columns:
        #     # print(daily_feat)
        #     # print(f'number of distinct {X[daily_feat].nunique()} values')
        #     self.last_avg_return = pd.DataFrame(pd.DataFrame(y_train[-5:]).mean())
        #     correlation_matrix = np.corrcoef(X_train[daily_feat], y_train)
        #     if (way_last_return>0 and correlation_matrix[0][1]>0):
        #         self.output_ways.append(1)
        #
        #     elif (way_last_return < 0 and correlation_matrix[0][1] < 0):
        #         self.output_ways.append(0)
        #     else:
        #         self.output_ways.append(0.5)

    def predict(self, X_test):
        # X_test = X_test.drop(self.columns_to_drop, axis=1)
        # self.last_avg_return = self.last_avg_return.append([self.last_avg_return] * (len(X_test) - 1), ignore_index=True)
        # next_way = np.max(self.output_ways)
        # if next_way>0:
        #     y_pred = self.last_avg_return
        # elif next_way<0:
        #     y_pred = 0*self.last_avg_return
        # else:
        #     y_pred=0.5*self.last_avg_return
        # return y_pred.values

        X_test = X_test.drop(self.columns_to_drop, axis=1)
        predictions = (X_test >= self.median_values).astype(int)

        if self.signal_type == 'lo':
            None
        if self.signal_type == 'ls':
            predictions = predictions.replace({0: -1})

        predictions = predictions.multiply(self.weigth_factors, axis=1)
        if self.mixing_type == 'sum':
            y_pred = predictions.sum(axis=1)

        if self.mixing_type == 'mean':
            y_pred = predictions.mean(axis=1)

        y_pred = y_pred.replace([np.inf, -np.inf], np.nan)
        y_pred = y_pred.fillna(0)
        return y_pred
    def get_features_importance(self, features_names):
        run_importances = {}
        # for (name, imp) in zip(features_names, self.model.coef_):
        #     run_importances[name] = imp
        return run_importances


#!/usr/bin/env python3
# coding: utf-8

import sqlite3

from multiprocessing import Pool
from napoleontoolbox.file_saver import dropbox_file_saver

from napoleontoolbox.file_saver import dropbox_file_saver
from napoleontoolbox.signal import signal_generator
from napoleontoolbox.connector import napoleon_connector
from napoleontoolbox.signal import signal_utility
from napoleontoolbox.parallel_run import signal_result_analyzer
from napoleontoolbox.parallel_run import launcher_utility
import json
import pandas as pd

class RealTimeBacktestSignalEnsemblingParalleLauncher():
    def __init__(self, starting_date=None, running_date=None, drop_token='', dropbox_backup=True, local_root_directory='../data',underlying = None, frequence='daily', selected_algo='',user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl',freqly_return_pkl_filename_suffix='freqly_candels.pkl'):
        self.starting_date = starting_date
        self.running_date = running_date
        self.args = []
        self.counter = 1
        self.seed = 0
        self.local_root_directory = local_root_directory

        self.underlying = underlying
        self.list_pkl_file_suffix = list_pkl_file_suffix
        self.frequence=frequence
        self.selected_algo=selected_algo
        self.dropbox_backup = dropbox_backup
        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
        self.dates_stub = self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y')

        self.list_pkl_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.list_pkl_file_suffix
        print('selected algos')
        print(self.list_pkl_file_name)
        if dropbox_backup:
            self.signals_list = self.dbx.download_pkl(pkl_file_name=self.list_pkl_file_name)
        else:
            self.signals_list = napoleon_connector.load_pickled_list(local_root_directory=self.local_root_directory,
                                                                 list_pkl_file_name=self.list_pkl_file_name)
        max_lookback_window = 0
        for me_signal in self.signals_list:
            self.args.append((me_signal))
            self.counter = self.counter + 1
            run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
            params = json.loads(run_json_string)
            if params['lookback_window']>max_lookback_window:
                max_lookback_window=params['lookback_window']
        self.max_lookback_window = max_lookback_window

        self.args.sort()
        self.user = user
        self.db_path_suffix = db_path_suffix
        self.filename =  user + db_path_suffix
        self.db_path = self.local_root_directory + self.filename
        self.runs = []
        self.totalRow = 0
        self.empty_runs_to_investigate = []
        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
        self.freqly_return_pkl_filename_suffix=freqly_return_pkl_filename_suffix
        self.saving_return_path = self.local_root_directory+self.underlying+self.freqly_return_pkl_filename_suffix
        freqly_df = pd.read_pickle(self.saving_return_path)
        print('number of duplicated dates')
        print(freqly_df.index.duplicated().sum())
        freqly_df = freqly_df.sort_index()
        print('time range before filtering ')
        print(max(freqly_df.index))
        print(min(freqly_df.index))
        # freqly_df = freqly_df[freqly_df.index >= self.starting_date]
        # freqly_df = freqly_df[freqly_df.index <= self.running_date]
        print('time range after filtering ')
        print(max(freqly_df.index))
        print(min(freqly_df.index))
        self.freqly_df = freqly_df

    def launchParallelPool(self, use_num_cpu):
        print('launching parallel computation for alphas at ')
        with Pool(processes=use_num_cpu) as pool:
            run_results = pool.starmap(self.runTrial, self.args)
        print('parallel computation done')
        print('results length')
        print(len(run_results))


    def launchSequential(self):
        print('launching sequential computation for alphas at ')
        run_results = []
        for meArg in self.args:
            run_results.append(self.runTrial(meArg))
        print('results length')
        print(len(run_results))


    def runTrial(self, me_signal):
        ## idiosyncratic run itself
        run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
        params = json.loads(run_json_string)
        signal_type = params['signal_type']
        normalization = params['normalization']
        trigger = params['trigger']
        transaction_costs = params['transaction_costs']
        if normalization and not signal_generator.is_signal_continuum(signal_type):
            return

        lookback_window = params['lookback_window']
        signal_generation_method_to_call = getattr(signal_generator, signal_type)
        last_generated_signal = signal_utility.compute_last_signal(self.freqly_df, lookback_window,
                                                                   lambda x: signal_generation_method_to_call(
                                                                       data=x, **params))
        print('last_generated_signal')
        print(last_generated_signal)
        return last_generated_signal


import collections, itertools
import nltk.classify.util, nltk.metrics
from nltk.classify import NaiveBayesClassifier
from nltk.corpus import movie_reviews, stopwords
from nltk.collocations import BigramCollocationFinder
from nltk.metrics import BigramAssocMeasures
from nltk.probability import FreqDist, ConditionalFreqDist


def evaluate_classifier(featx):
    negids = movie_reviews.fileids('neg')
    posids = movie_reviews.fileids('pos')

    negfeats = [(featx(movie_reviews.words(fileids=[f])), 'neg') for f in negids]
    posfeats = [(featx(movie_reviews.words(fileids=[f])), 'pos') for f in posids]

    negcutoff = len(negfeats) * 3 / 4
    poscutoff = len(posfeats) * 3 / 4

    trainfeats = negfeats[:negcutoff] + posfeats[:poscutoff]
    testfeats = negfeats[negcutoff:] + posfeats[poscutoff:]

    classifier = NaiveBayesClassifier.train(trainfeats)
    refsets = collections.defaultdict(set)
    testsets = collections.defaultdict(set)

    for i, (feats, label) in enumerate(testfeats):
        refsets[label].add(i)
        observed = classifier.classify(feats)
        testsets[observed].add(i)

    print('accuracy:', nltk.classify.util.accuracy(classifier, testfeats))
    print('pos precision:', nltk.metrics.precision(refsets['pos'], testsets['pos']))
    print('pos recall:', nltk.metrics.recall(refsets['pos'], testsets['pos']))
    print('neg precision:', nltk.metrics.precision(refsets['neg'], testsets['neg']))
    print('neg recall:', nltk.metrics.recall(refsets['neg'], testsets['neg']))
    classifier.show_most_informative_features()




#!/usr/bin/env python
# coding: utf-8

from abc import ABC, abstractmethod

import pandas as pd
import numpy as np

from napoleontoolbox.file_saver import dropbox_file_saver
from napoleontoolbox.signal import signal_generator
from napoleontoolbox.connector import napoleon_connector
from napoleontoolbox.signal import signal_utility
from napoleontoolbox.parallel_run import signal_result_analyzer
from napoleontoolbox.parallel_run import launcher_utility
import json

from pathlib import Path

class AggregatingSignalAnalyzer():
        def __init__(self, calibration_starting_date=None, calibration_running_date=None, starting_date=None, running_date=None,
                 drop_token=None, dropbox_backup=True, underlying=None, frequence='daily', selected_algo='', algo='',
                 freqly_return_pkl_filename_suffix='freqly_candels.pkl', aggregated_pkl_file_suffix='freqly_to_mix.pkl',
                 aggregated_pkl_mapping_file_suffix='freqly_to_mix_mapping.pkl', list_pkl_file_suffix='my_list.pkl',
                 local_root_directory='../data/', user='napoleon', add_turn_over=False, skipping_size=200,
                 recompute_all=True, db_path_suffix=None, daily_ia_lookback_window=21):
            super().__init__()
            self.starting_date = starting_date
            self.running_date = running_date
            self.calibration_starting_date = calibration_starting_date
            self.calibration_running_date = calibration_running_date

            self.freqly_return_pkl_filename_suffix = freqly_return_pkl_filename_suffix
            self.aggregated_pkl_file_suffix = aggregated_pkl_file_suffix
            self.aggregated_pkl_mapping_file_suffix = aggregated_pkl_mapping_file_suffix
            self.underlying = underlying
            self.list_pkl_file_suffix = list_pkl_file_suffix
            self.frequence = frequence
            self.algo = algo
            self.selected_algo = selected_algo

            self.calibrating_dates_stub = self.calibration_starting_date.strftime(
                '%d_%b_%Y') + '_' + self.calibration_running_date.strftime('%d_%b_%Y')
            self.dates_stub = self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y')

            self.list_pkl_file_name = self.calibrating_dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.list_pkl_file_suffix
            self.aggregated_pkl_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.aggregated_pkl_file_suffix
            self.aggregated_pkl_mapping_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.aggregated_pkl_mapping_file_suffix

            self.local_root_directory = local_root_directory
            self.user = user
            self.dropbox_backup = dropbox_backup
            self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token, dropbox_backup=dropbox_backup)
            self.running_date = running_date
            self.starting_date = starting_date
            self.add_turn_over = add_turn_over
            self.skipping_size = skipping_size
            self.signals_list = napoleon_connector.load_pickled_list(local_root_directory=self.local_root_directory,
                                                                     list_pkl_file_name=self.list_pkl_file_name)

            self.signals_list = list(self.signals_list)
            self.signals_list.sort()

            self.dates_stub = starting_date.strftime('%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y')
            self.saving_return_path = self.local_root_directory + self.underlying + '_' + self.dates_stub + self.freqly_return_pkl_filename_suffix

            self.recompute_all = recompute_all
            self.db_path_suffix = db_path_suffix
            self.db_path_name = launcher_utility.get_database_name(user, frequence, underlying, algo, starting_date,
                                                                   running_date, db_path_suffix)

            # if not self.recompute_all:
            #     saver = signal_result_analyzer.ParallelRunResultAnalyzer(drop_token=drop_token,
            #                                                              local_root_directory=local_root_directory, user=user,
            #                                                              db_path_name=self.db_path_name)
            #     self.precomputed_results_df, self.precomputed_signals_df = saver.analyzeAllRunResults()
            # else:
            #     self.precomputed_results_df, self.precomputed_signals_df  = None, None

            if not self.recompute_all:
                print('prereading data from ' + self.db_path_name)
                saver = signal_result_analyzer.ParallelRunResultAnalyzer(drop_token=drop_token,
                                                                         local_root_directory=local_root_directory, user=user,
                                                                         db_path_name=self.db_path_name)
                self.precomputed_results_df, self.precomputed_signals_df = saver.analyzeAllRunResults()
                self.daily_ia_lookback_window = daily_ia_lookback_window
            print('initialization done')


        def load_aggregated_results(self):
            print(f'reading results saved to {self.local_root_directory + self.aggregated_pkl_file_name}')
            cumulated_signals_df = pd.read_pickle(self.local_root_directory + self.aggregated_pkl_file_name)
            print(f'reading results saved to {self.local_root_directory + self.aggregated_pkl_mapping_file_name}')
            mapping_dictionary_df = pd.read_pickle(self.local_root_directory + self.aggregated_pkl_mapping_file_name)
            return cumulated_signals_df, mapping_dictionary_df

